/* (C) Copyright 1997 Albert Ludwigs University Freiburg
 *     Institute of Computer Science
 *
 * All rights reserved. Use of this software is permitted for 
 * non-commercial research purposes, and it may be copied only 
 * for that use.  All copies must include this copyright message.
 * This software is made available AS IS, and neither the authors
 * nor the  Albert Ludwigs University Freiburg make any warranty
 * about the software or its performance. 
 */
/* 	$Id: not_preproc.c,v 1.7 1998/10/28 23:40:36 wolf Exp $	 */

#ifndef lint
static char vcid[] = "$Id: not_preproc.c,v 1.7 1998/10/28 23:40:36 wolf Exp $";
#endif /* lint */

#include "ipp.h"

/*
 * function prototypes
 */
token_list get_types_from_constlist(fact_list pred, fact_list var);
token_list copy_token_list(token_list source);
void append_to_effects(token name, token_list args, fact_list* effect);
void build_neg_predlist(fact_list pred, token_list f_types);
void store_fact_list(fact_list i, fact_list* start, fact_list* last);
void store_token_list(token_list i, token_list* start, token_list* last);
int not_in_list( token_list pred, fact_list list );

void print_factlist(fact_list list);

token_list get_objects_of_type( token type_name )
{
  fact_list objs;

  for ( objs = global_object_fl; objs; objs = objs->next )
    {
      if ( strcmp( objs->item->item, type_name ) == SAME )
	/* found list of objects with type type_name, now
	   set objs to the first one */
	break;
    }
  if ( objs )
    return objs->item->next;
  else 
    return NULL;
}

/* uses all objects of a type attaching them to the end
   of every (yet incompletely instantiated) list of 
   predicates. Same method as in rec_build_exgoal_fl() */
fact_list rec_inst_pred( fact_list facts, token_list typelist )
{
  /* facts is a list of incomplete token_lists, which
     is splitted and multiplied so that all objects
     of the next type needed can be concatenated */
  token_list o, objs = NULL;
  token_list end = NULL;
  fact_list t=NULL;
  fact_list result=NULL;

  if ( !typelist )
    return facts;

  /* first search objects of type 'typelist->item'... 
     'types' is a global fact_list built during fact file scanning
     using build_type_list(), each token_list begins
     with a type name followed by all objects of that type */
  for ( t=global_object_fl; t; t=t->next )
    if ( strcmp( t->item->item, typelist->item ) == SAME )
      /* if the list beginning with the
	 right type name is found, continue with 2nd
	 element */
      objs = t->item->next;

  if ( !objs )
    { /* no objects of type typelist->item were found, which may mean
	 that there is a typing error or that there are no objects of
	 that type (which will lead to uninstatiability of the operator) */
      return NULL;
    }
  /* ...then split and copy facts and build a new list for every 
     possible next parameter instatiation */
  result = t = new_fact_list();

  for ( ; facts; facts=facts->next )
    {
      for ( o=objs; o; o=o->next )
	{
	  t->next = new_fact_list();
	  t->next->item = copy_tl_return_end( facts->item, &end );
	  end->next = new_token_list();
	  end->next->item = o->item;
	  t = t->next;
	}
    } 
  return rec_inst_pred( result->next, typelist->next );
}


/* returns a fact_list in which each token_list is one
   fully instantiated predicate */
/* Note: SHARES PRED STRING (name)!! */
fact_list instantiate_pred( token name, token_list typelist, 
					   char * init_token )
{
  fact_list result;
  
  result = new_fact_list();
  result->item = new_token_list();
  result->item->item = name;

  /* tack on the leading token (prob EFFECT_STR or PRECOND_STR) */
  if (init_token) {
    token_list temp = new_token_list();
    temp->item = new_token(strlen(init_token) + 1);
    strcpy(temp->item, init_token);
    temp->next = result->item;
    result->item = temp;
  }

  /* frank memory leak, result geht verloren.
   * es wuerde genuegen nur name zu uebergeben, aber die
   * Funktion rec_inst_pred() braucht eine fact_list, da
   * in rec_inst_pred() die Funktion copy_tl_return_end
   * aufgerufen wird und diese eine fact_list braucht.
   * copy_tl_return_end wird auch von anderen Funktionen
   * benutzt und kann nicht so einfach geaendert werden
   */
  result = rec_inst_pred( result, typelist );

  return result;
}

/* 
 * All predicates that appear negated are fully instantiated and
 * then they are inserted into the orig_initial_facts(global),
 * if they do not appear allready not negated.
 *
 * returns: nothing
 */
void transform_initials()
{
  fact_list i_list=NULL;
  fact_list pred=NULL;
  fact_list new_initial;

  for (pred = predicates_list; pred; pred = pred->next)
    {
      /* get a list with all possible instatiations of actual predicate */
      i_list = instantiate_pred( pred->item->item,
						   pred->item->next,
						   EFFECT_STR );
      
      /* now step through the list, find those facts that
	    are not in initial state and add their negation */
      while (i_list)
	   {
		/* include negations of those instations that do not appear
		   in the initial fact list */
		if (not_in_list(i_list->item, orig_initial_facts))
		  {
		    /* create negated form of i_list and insert into 
			* orig_initial_facts
			*/
		    new_initial = new_fact_list();
		    new_initial->item = new_token_list();
		    new_initial->item->item = 
			 new_token(strlen(i_list->item->item) + 1 + 1);

		    sprintf(new_initial->item->item, 
				  "%s%s", "!", i_list->item->item);

		    new_initial->item->next = i_list->item->next;

		    new_initial->next = orig_initial_facts;
		    orig_initial_facts = new_initial;

		    /* 	      my_free(i_list->item->item); */
		    i_list = i_list->next;
		  }
		else
		  {
		    /* 	      old_i_list = i_list; */
		    i_list = i_list->next;
		    /* 	      my_free(old_i_list); */
		  }
	   }
    }
}

/* Checks if predicate is already in list, 
 * we have to check the types again...
 *
 * pred: the predicate, we search for
 *       the first token is the name and the following are the arguments
 * list: the list where we search in
 *       every item is a token_list like the above
 *
 * returns: TRUE if the item is not in the list
 */
int not_in_list( token_list pred, fact_list list )
{
  return in_list( pred, list ) == NULL;
}

fact_list in_list( token_list pred, fact_list list )
{
  fact_list which_fact = NULL;
  fact_list fact;             /* helpers to traverse lists */
  token_list t_list;
  token_list f_pred;

  /* traverse the list where we search a predicate */
  for (fact = list; fact; fact = fact->next)
    {
      /* use a helper variable to keep pred */
      f_pred = pred;

      /* traverse the items of the fact_lists */
      for (t_list = fact->item; t_list && f_pred; f_pred = f_pred->next, t_list = t_list->next)
	   {
		if (strcmp(t_list->item, f_pred->item) != SAME)
		  {
		    /* we didn't find it, stop */
		    break;
		  }
	   }
	 if (f_pred == NULL && t_list == NULL) {
	   /* found it! */
	   which_fact = fact;
	   break;
	 }
    }

  return which_fact;
}

