// $Id: ClPoint.h,v 1.1 1998/11/28 17:52:36 wolf Exp $
//
// Cassowary Incremental Constraint Solver
// Original Smalltalk Implementation by Alan Borning
// This C++ Implementation by Greg J. Badros, <gjb@cs.washington.edu>
// http://www.cs.washington.edu/homes/gjb
// (C) 1998 Alan Borning and Greg Badros
// See ../COPYRIGHT for legal details regarding this software
//
// ClPoint.h

#ifndef ClPoint_H
#define ClPoint_H

#include "Cassowary.h"
#include "ClVariable.h"

// ClPoint is just a convenience class for pairs of
// ClVariables -- often useful for coordinate pairs in 2-space
class ClPoint {
 public:
  ClPoint(Number x, Number y)
    : _clv_x(x), _clv_y(y)
    { }

  ClPoint()
    { }

  ClVariable &X()
    { return _clv_x; }

  ClVariable &Y()
    { return _clv_y; }

  const ClVariable &X() const
    { return _clv_x; }

  const ClVariable &Y() const
    { return _clv_y; }

  void SetXY(Number x, Number y)
    { _clv_x.set_value(x); _clv_y.set_value(y); }

  Number Xvalue() const
    { return X().value(); }

  Number Yvalue() const
    { return Y().value(); }

 private:
  ClVariable _clv_x;
  ClVariable _clv_y;

  friend ostream &operator<<(ostream &xo, const ClPoint &clp);
  
};

inline ostream &
operator<<(ostream &xo, const ClPoint &clp)
{
  xo << "(" << clp._clv_x << ", " << clp._clv_y << ")";
  return xo;
}

#endif
